/*
 * Copyright (c) Huawei Technologies Co., Ltd. 2020-2024. All rights reserved.
 */

#ifndef DPC_IMPL_TYPE_LINEGEOMETRY_H
#define DPC_IMPL_TYPE_LINEGEOMETRY_H

#include <cfloat>
#include <cmath>
#include "impl_type_float.h"

namespace dpc {
struct LineGeometry {
    ::Float c0{ 0.0F };
    ::Float c1{ 0.0F };
    ::Float c2{ 0.0F };
    ::Float c3{ 0.0F };
    ::Float start_range{ 0.0F };
    ::Float end_range{ 0.0F };

    static bool IsPlane()
    {
        return true;
    }


    using IsEnumerableTag = void;
    template<typename F>
    void enumerate(F& fun)
    {
        fun(c0);
        fun(c1);
        fun(c2);
        fun(c3);
        fun(start_range);
        fun(end_range);
    }

    template<typename F>
    void enumerate(F& fun) const
    {
        fun(c0);
        fun(c1);
        fun(c2);
        fun(c3);
        fun(start_range);
        fun(end_range);
    }

    template<typename F>
    void enumerate_internal(F& fun)
    {
        fun("c0", c0);
        fun("c1", c1);
        fun("c2", c2);
        fun("c3", c3);
        fun("start_range", start_range);
        fun("end_range", end_range);
    }

    template<typename F>
    void enumerate_internal(F& fun) const
    {
        fun("c0", c0);
        fun("c1", c1);
        fun("c2", c2);
        fun("c3", c3);
        fun("start_range", start_range);
        fun("end_range", end_range);
    }

    friend bool operator==(const ::dpc::LineGeometry& lhs, const ::dpc::LineGeometry& rhs) noexcept
    {
        return (fabs(static_cast<double>(lhs.c0 - rhs.c0)) < DBL_EPSILON) && (fabs(static_cast<double>(lhs.c1 - rhs.c1)) < DBL_EPSILON) && (fabs(static_cast<double>(lhs.c2 - rhs.c2)) < DBL_EPSILON) && (fabs(static_cast<double>(lhs.c3 - rhs.c3)) < DBL_EPSILON) && (fabs(static_cast<double>(lhs.start_range - rhs.start_range)) < DBL_EPSILON) && (fabs(static_cast<double>(lhs.end_range - rhs.end_range)) < DBL_EPSILON);
    }
};
} // namespace dpc


#endif // DPC_IMPL_TYPE_LINEGEOMETRY_H
